@@ -13,4 +13,4 @@
 #undef WITH_REGEX
 #undef HAVE_GTKTEXT_PATCH
 #undef USE_FORK_CODE
-#undef HAVE_LIBGUILE
+#undef HAVE_GUILE
