@@ -1,4 +1,4 @@
-/* $Header: /cvs/gnome/gIDE/src/gI_about.h,v 1.3 1999/11/06 17:45:38 jpr Exp $ */
+/* $Header: /cvs/gnome/gIDE/src/gI_about.h,v 1.4 2000/04/16 04:59:56 jpr Exp $ */
 /* gIDE
  * Copyright (C) 1998-2000 Steffen Kern
  *
@@ -20,7 +20,7 @@
 #ifndef GI_ABOUT_H
 #define GI_ABOUT_H
 
-#include <gtk/gtk.h>
+#include <gnome.h>
 
 /*
  * Prototypes for 'gI_about.c'
