@@ -1,4 +1,4 @@
-/* $Header: /cvs/gnome/gIDE/src/gI_common.c,v 1.2 1999/11/06 17:45:38 jpr Exp $ */
+/* $Header: /cvs/gnome/gIDE/src/gI_common.c,v 1.7 2000/04/16 04:59:56 jpr Exp $ */
 /* gIDE
  * Copyright (C) 1998-2000 Steffen Kern
  *
@@ -17,8 +17,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <gtk/gtk.h>
-#include <gnome.h>
+#include "gide.h"
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -31,45 +30,26 @@
 #include <grp.h>
 #include <sys/types.h>
 #include <pwd.h>
-#include "structs.h"
+
 #include "gI_common.h"
 
 /* local prototypes */
 static void modal_dialog_callback(GtkWidget *w,gpointer );
 
-gI_document *gI_document_get_current( gI_window *window );
+GideDocument *gI_window_get_current_doc( GideWindow *window );
 
 
 gchar *get_path( gchar *filename )
 {
-	gchar *ptr;
 	gchar *str;
-	gchar *retstr;
 
-	/* allocate some memory, so the user can free it later */
-	if( !filename )
-	{
-		str = g_malloc( 1 );
-		str[0] = 0;
-		return( str );
+	str = g_dirname(filename);
+	if (!strcmp(str, ".")) {
+		free(str);
+		str = g_strdup("");
 	}
 
-	str = g_strdup( filename );
-
-	ptr = strrchr( str, '/' );
-	if( ptr )
-	{
-		*ptr = '\0';
-		retstr = g_strdup( str );
-		*ptr = 'x';
-		g_free( str );
-		return( retstr );
-	}
-	else
-	{
-		str[0]=0;     /* no path dont free up like before else there will be to mush rewrite*/
-		return str;   /* let user free it up for herselft */
-	}
+	return str;
 }
 
 
@@ -110,6 +90,8 @@ gint gI_ask_dialog( gchar *msg )
 	                             NULL );
 
 	gtk_window_set_modal( GTK_WINDOW( dlg ), TRUE );
+	gnome_dialog_set_parent( GNOME_DIALOG( dlg ),
+	                         GTK_WINDOW( main_window ) );
 
 	return( gnome_dialog_run_and_close( GNOME_DIALOG( dlg ) ) );
 }
@@ -126,7 +108,8 @@ void gI_error_dialog( gchar *msg)
 {
 	GtkWidget *dlg;
 
-	dlg = gnome_error_dialog( msg );
+	dlg = gnome_error_dialog_parented( msg,
+	                                   GTK_WINDOW( main_window ) );
 
 	gnome_dialog_run_and_close( GNOME_DIALOG( dlg ) );
 }
@@ -143,7 +126,8 @@ void gI_ok_dialog( gchar *msg)
 {
 	GtkWidget *dlg;
 
-	dlg = gnome_ok_dialog( msg);
+	dlg = gnome_ok_dialog_parented( msg,
+	                                GTK_WINDOW( main_window ) );
 
 	gnome_dialog_run_and_close( GNOME_DIALOG( dlg ) );
 }
@@ -354,7 +338,7 @@ GtkWidget *entry_dialog( gchar *label, gchar *title, void *ok_func )
 }
 
 
-glong get_last_mod( gchar *filename )
+glong get_last_mod( gchar const *filename )
 {
 	struct stat f_stat;
 
@@ -618,8 +602,8 @@ GList *gI_file_sel_new( const gchar *title, glong file_ops, glong multi )
 {
 	GtkWidget *filesel = NULL;
 	glong *selected;
-	gI_document *current;
-	extern gI_window *main_window;
+	GideDocument *current;
+	extern GideWindow *main_window;
 
 	selected = (glong *) malloc( sizeof( glong ) );
 
@@ -643,7 +627,7 @@ GList *gI_file_sel_new( const gchar *title, glong file_ops, glong multi )
 	gtk_widget_show( filesel );
 	gtk_grab_add( filesel );
 
-	if( (current = gI_document_get_current( main_window )) )
+	if( (current = gI_window_get_current_doc( main_window )) )
 	{
 		gchar *path;
 
@@ -764,4 +748,27 @@ void gI_file_sel_free_list( GList *filenames )
 	g_list_free( filenames );
 }
 
+gint
+strcase_equal (gconstpointer v, gconstpointer v2)
+{
+        return strcasecmp ((const gchar*) v, (const gchar*)v2) == 0;
+}
+
+guint
+strcase_hash (gconstpointer v)
+{ 
+        const unsigned char *s = (const unsigned char *)v;
+        const unsigned char *p;
+        guint h = 0, g; 
+   
+        for(p = s; *p != '\0'; p += 1) {
+                h = ( h << 4 ) + tolower (*p);
+                if ( ( g = h & 0xf0000000 ) ) {  
+                        h = h ^ (g >> 24);
+                        h = h ^ g;
+                }
+        }
+ 
+        return h /* % M */;
+}
 
