@@ -1,4 +1,4 @@
-/* $Header: /cvs/gnome/gIDE/src/gI_compile.c,v 1.3 1999/12/05 19:10:48 jpr Exp $ */
+/* $Header: /cvs/gnome/gIDE/src/gI_compile.c,v 1.9 2000/04/22 17:23:01 jpr Exp $ */
 /* gIDE
  * Copyright (C) 1998-2000 Steffen Kern
  *
@@ -17,10 +17,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <gtk/gtk.h>
-#include <gdk/gdkx.h>
+#include "gide.h"
 #include <gdk/gdkprivate.h>
-#include <gnome.h>
 #include <unistd.h>
 #include <time.h>
 #include <string.h>
@@ -34,15 +32,12 @@
 #include <errno.h>
 #include <ctype.h>
 
-#include "structs.h"
 #include "gI_compile.h"
-#include "gI_document.h"
 #include "gI_cfp.h"
 #include "gI_project.h"
 #include "gI_edit.h"
 #include "gI_common.h"
 #include "gI_file.h"
-#include "gI_window.h"
 #include "gI_prefs.h"
 
 
@@ -55,9 +50,14 @@ static GList *pid_data = NULL;
 static long canceled = 0;
 #endif
 
-/* externs */
-extern gI_window *main_window;
-extern gI_config *cfg;
+
+/* local prototypes */
+static void window_deiconify( GdkWindow *window );
+static gboolean window_is_iconified( GdkWindow *window );
+#ifdef USE_FORK_CODE
+static void sigchld_handler( gint signal );
+#endif
+static gint gI_exec( gchar **argv, gchar *socket );
 
 
 /*
@@ -71,11 +71,11 @@ gI_compile_window *gI_compile_window_new( gchar *title )
 {
 	gI_compile_window *compile_window;
 	GtkWidget *vbox;
-	gI_document *current;
+	GideDocument *current;
 	GtkWidget *hbox;
 	GtkWidget *scrolled_win;
 
-	current = gI_document_get_current( main_window );
+	current = gI_window_get_current_doc( main_window );
 	if( !current )
 		return( NULL );
 
@@ -123,7 +123,7 @@ gI_compile_window *gI_compile_window_new( gchar *title )
 }
 
 
-void gI_compile_window_destroy( GtkWidget *widget, gI_document *document )
+void gI_compile_window_destroy( GtkWidget *widget, GideDocument *document )
 {
 	if( !document
 	        || !document->compile_window
@@ -161,7 +161,7 @@ void gI_compile_window_destroy( GtkWidget *widget, gI_document *document )
 }
 
 
-void window_deiconify( GdkWindow *window )
+static void window_deiconify( GdkWindow *window )
 {
 	GdkWindowPrivate *Private;
 
@@ -170,7 +170,7 @@ void window_deiconify( GdkWindow *window )
 }
 
 
-gboolean window_is_iconified( GdkWindow *window )
+static gboolean window_is_iconified( GdkWindow *window )
 {
 	XWindowAttributes xattr;
 	GdkWindowPrivate *Private;
@@ -373,7 +373,7 @@ void update_compile_list( udata *update_data )
 
 
 #ifdef USE_FORK_CODE
-void sigchld_handler( gint signal )
+static void sigchld_handler( gint signal )
 {
 	gint status;
 	udata *update_data;
@@ -426,7 +426,7 @@ void sigchld_handler( gint signal )
 }
 
 
-gint gI_exec( gchar **argv, gchar *socket )
+static gint gI_exec( gchar **argv, gchar *socket )
 {
 	gint pid;
 	gint ret;
@@ -573,7 +573,7 @@ void build_comp_string_fork( gchar *str, gchar **argv )
 
 
 
-static void _compile_compile( gI_document *document, glong type )
+static void _compile_compile( GideDocument *document, glong type )
 {
 	gchar exec_string[MAXLEN] = "";
 	gchar tmp_string[STRLEN];
@@ -667,16 +667,13 @@ static void _compile_compile( gI_document *document, glong type )
 
 }
 
-
-void compile_compile( GtkWidget *widget, gpointer data )
+static void
+compile_compile( GideWindow *window, glong type )
 {
-	gI_document *current;
-	glong type;
-
-	type = (glong) data;
+	GideDocument *current;
 
 	/* get current page */
-	current = gI_document_get_current( main_window );
+	current = gI_window_get_current_doc( window );
 
 	if( current->working )
 		return;
@@ -701,6 +698,19 @@ void compile_compile( GtkWidget *widget, gpointer data )
 	/*current->working = 0;*/
 }
 
+void
+compile_compile_exec( GtkWidget *widget, gpointer data )
+{
+	g_assert(IS_GIDE_WINDOW(data));
+	compile_compile( GIDE_WINDOW(data), COMPILE_EXEC );
+}
+
+void
+compile_compile_obj( GtkWidget *widget, gpointer data )
+{
+	g_assert(IS_GIDE_WINDOW(data));
+	compile_compile( GIDE_WINDOW(data), COMPILE_OBJ );
+}
 
 /*
  ---------------------------------------------------------------------
@@ -712,7 +722,7 @@ void compile_compile( GtkWidget *widget, gpointer data )
 void compile_make( GtkWidget *widget, gpointer data )
 {
 	gchar tmp_string[STRLEN];
-	gI_document *current;
+	GideDocument *current;
 	gchar *path;
 	gint pid = -1;
 	udata *update_data;
@@ -722,9 +732,11 @@ void compile_make( GtkWidget *widget, gpointer data )
 #else
 	gchar exec_string[MAXLEN];
 #endif
+	gI_project*							project;
+	gchar*								make_command;
 
 	/* get current page */
-	current = gI_document_get_current( main_window );
+	current = gI_window_get_current_doc( main_window );
 	if(!current) 
 		return;
 		
@@ -737,26 +749,82 @@ void compile_make( GtkWidget *widget, gpointer data )
 
 	current->working = 1;
 
+	/* create make commandline */
+#ifdef USE_FORK_CODE
+	argv[0] = cfg->make;
+	argv[1] = NULL;
+#endif
+	if((project = gI_project_get_current()))
+	{
+		gchar*							mf;
+
+		if(!isempty(project->prjroot))
+		{
+			/* change working dir */
+			chdir( project->prjroot );
+		}
+
+		if(!isempty(project->version))
+		{
+			mf = g_strconcat(project->prjroot, "/Makefile", ".", project->name,
+				"-", project->version, NULL);
+		}
+		else
+		{
+			mf = g_strconcat(project->prjroot, "/Makefile", ".", project->name,
+				NULL);
+		}
+		if(!file_exist(mf))
+		{
+			if(!file_exist("Makefile"))
+			{
+				gI_error_dialog(_("Sorry, no makefile present!"));
+				g_free(mf);
+				return;
+			}
+			make_command = g_strdup(cfg->make);
+		}
+		else
+		{
+			make_command = g_strdup_printf("%s -f %s", cfg->make, mf);
+#ifdef USE_FORK_CODE
+			argv[1] = "-f";
+			argv[2] = g_strdup(mf);
+			argv[3] = NULL;
+#endif
+		}
+		g_free(mf);
+	}
+	else
+	{
+		/* if there's a file open, get the path from it and try make there */
+		if( current->filename != NULL )
+		{
+			/* change working dir */
+			path = get_path( current->filename );
+			if( !isempty( path ) )
+			{
+				chdir( path );
+				g_free( path );
+			}
+		}
+
+		if(!file_exist("Makefile"))
+		{
+			gI_error_dialog(_("Sorry, no makefile present!"));
+			return;
+		}
+		make_command = g_strdup(cfg->make);
+	}
+
 	if( current->compile_window == NULL )
 	{
-		current->compile_window = gI_compile_window_new( _("Make Window") );
+		current->compile_window = gI_compile_window_new( _("Make window") );
 		gtk_widget_show( current->compile_window->window );
 	}
 
 	current->compile_window->pid = pid;
 
-	/* if there's a file open, get the path from it and try make there */
-	if( current->filename != NULL )
-	{
-		/* change working dir */
-		path = get_path( current->filename );
-		if( !isempty( path ) )
-		{
-			chdir( path );
-			g_free( path );
-		}
-	}
-
 	/* generate a *really* unique filename */
 	g_snprintf( filename, STRLEN, "%s/gide_make.%d.%d", cfg->tmpdir,
 	            (gint) time(0), (gint)getpid() );
@@ -774,22 +842,27 @@ void compile_make( GtkWidget *widget, gpointer data )
 	/* show compiling dialog */
 	show_compiling( current, current->compile_window );
 
-	/* start compile */
 #ifdef USE_FORK_CODE
-	argv[0] = cfg->make;
-	argv[1] = NULL;
-
 	pid = gI_exec( argv, filename );
+
+	if(argv[1] != NULL)
+	{
+		g_free(argv[2]);
+	}
 #else
 	/* build exec string */
-	g_snprintf( exec_string, STRLEN, "%s -c \"(%s 1>%s 2>&1 ; echo \"Done.\" >%s) &\"", cfg->bash, cfg->make, filename, filename );
+	g_snprintf( exec_string, STRLEN,
+		"%s -c \"(%s 1>%s 2>&1 ; echo \"Done.\" >%s) &\"",
+		cfg->bash, make_command, filename, filename );
 
 	pid = system( exec_string );
 #endif
 
-	items[0] = cfg->make;
+	items[0] = make_command;
 	gtk_clist_append( GTK_CLIST( current->compile_window->list ), items );
 
+	g_free(make_command);
+
 	current->compile_window->pid = pid;
 
 	/* try to open socket */
@@ -809,7 +882,8 @@ void compile_make( GtkWidget *widget, gpointer data )
 	/*fgets( buf, sizeof(buf), update_data->file );
 	*current->compile_window->pid = atoi( buf );*/
 
-	update_data->input_id = gdk_input_add( update_data->file, GDK_INPUT_READ, (GdkInputFunction) update_compile_list, (gpointer) update_data );
+	update_data->input_id = gdk_input_add( update_data->file, GDK_INPUT_READ,
+		(GdkInputFunction) update_compile_list, (gpointer) update_data );
 
 #ifdef USE_FORK_CODE
 	table_assoc( pid, update_data );
@@ -825,7 +899,7 @@ void compile_link( GtkWidget *widget, gpointer data )
 	gchar tmp_string[STRLEN];
 	gchar exec_file[STRLEN];
 	gchar *ptr;
-	gI_document *current;
+	GideDocument *current;
 	gchar *items[1];
 	udata *update_data;
 	gint pid = -1;
@@ -834,7 +908,7 @@ void compile_link( GtkWidget *widget, gpointer data )
 	glong i;
 #endif
 
-	current = gI_document_get_current( main_window );
+	current = gI_window_get_current_doc( main_window );
 	if( !current )
 		return;
 
@@ -946,9 +1020,8 @@ void compile_link( GtkWidget *widget, gpointer data )
 
 }
 
-
-
-void show_compiling( gI_document *current, gI_compile_window *compile_window )
+void
+show_compiling( GideDocument *current, gI_compile_window *compile_window )
 {
 	GtkWidget *vbox;
 	GtkWidget *label;
@@ -990,7 +1063,7 @@ void show_compiling( gI_document *current, gI_compile_window *compile_window )
 
 
 #ifdef USE_FORK_CODE
-void cancel_compiling( GtkWidget *widget, gI_document *current )
+void cancel_compiling( GtkWidget *widget, GideDocument *current )
 {
 	if( !current )
 	{
@@ -1028,7 +1101,7 @@ void cancel_compiling( GtkWidget *widget, gI_document *current )
 #endif
 
 
-static void _check_file_open( gI_document *document, gchar *filename )
+static void _check_file_open( GideDocument *document, gchar *filename )
 {
 	if( document )
 	{
@@ -1056,15 +1129,15 @@ gint check_file_open( gchar *filename )
 gint get_point_from_line( gint tline )
 {
 	gulong line,point,i,length;
-	gI_document *current;
+	GideDocument *current;
 	gchar *ch;
 
 	line=1;
 	point=1;
 
-	current = gI_document_get_current( main_window );
+	current = gI_window_get_current_doc( main_window );
 
-	length = gI_text_get_length( current->text );
+	length = gI_text_get_length( current );
 
 	if( length > 0 )
 	{
@@ -1074,7 +1147,7 @@ gint get_point_from_line( gint tline )
 
 	for(i=1;i<=length;i++)
 	{
-		ch = gtk_editable_get_chars( GTK_EDITABLE( current->text ), i-1, i );
+		ch = gtk_editable_get_chars( GTK_EDITABLE( current ), i-1, i );
 		if( ch[0] == '\n' )
 		{
 			g_free( ch );
@@ -1094,8 +1167,8 @@ gint get_point_from_line( gint tline )
 
 	if( point < 0 )
 		point = 0;
-	if( point > gI_text_get_length( current->text ) )
-		point = gI_text_get_length( current->text );
+	if( point > gI_text_get_length( current ) )
+		point = gI_text_get_length( current );
 
 	return( point );
 }
@@ -1104,7 +1177,7 @@ gint get_point_from_line( gint tline )
 gint get_line_from_point( gint tpoint )
 {
 	gint line,point,i,length;
-	gI_document *current;
+	GideDocument *current;
 	gchar *c = NULL;
 
 	/* Init */
@@ -1112,15 +1185,15 @@ gint get_line_from_point( gint tpoint )
 	point=0;
 
 	/* get current document */
-	current = gI_document_get_current( main_window );
+	current = gI_window_get_current_doc( main_window );
 
 	/* get length of current document */
-	length = gI_text_get_length( current->text );
+	length = gI_text_get_length( current );
 
 	/* read... */
 	for(i=1;i<=length;i++)
 	{
-		c =  gtk_editable_get_chars( GTK_EDITABLE( current->text ), i-1, i );
+		c =  gtk_editable_get_chars( GTK_EDITABLE( current ), i-1, i );
 		if( *c == '\n' )
 		{
 			if( point >= tpoint )
@@ -1148,18 +1221,18 @@ hit:
 glong goto_file( gchar *filename )
 {
 	glong nod,i,found = 0,last;
-	gI_document *current;
+	GideDocument *current;
 
 	/* reimplement this stuff!! */
 
 	last = gtk_notebook_get_current_page( GTK_NOTEBOOK( main_window->notebook ) );
 
-	nod = gI_document_get_no( main_window );
+	nod = gI_window_num_docs( main_window );
 
 	for(i=0;i<nod;i++)
 	{
 		gtk_notebook_set_page( GTK_NOTEBOOK( main_window->notebook ), i );
-		current = gI_document_get_current( main_window );
+		current = gI_window_get_current_doc( main_window );
 		if( current->filename != NULL )
 		{
 			if( !strcmp( current->filename, filename ) )
@@ -1186,17 +1259,17 @@ glong goto_file( gchar *filename )
  */
 void goto_point( gint line )
 {
-	gI_document *current;
+	GideDocument *current;
 	int point;
 	int i;
 	glong changed;
 	double tl, ln, adjval;
 
-	current = gI_document_get_current( main_window );
+	current = gI_window_get_current_doc( main_window );
 	if( !current )
 		return;
 
-	tl = (double) get_total_lines( current->text );
+	tl = (double) get_total_lines( GTK_WIDGET(current) );
 	ln = (double) line;
 
 	if( tl < 10.0 )
@@ -1211,12 +1284,12 @@ void goto_point( gint line )
 		return;
 	}
 
-	gI_text_set_point( current->text, get_point_from_line( line ) );
+	gI_text_set_point( current, get_point_from_line( line ) );
 
 	/* borrowed from gEdit */
-	adjval = (ln * GTK_ADJUSTMENT( gI_text_vadj(current->text ) )->upper ) / tl - GTK_ADJUSTMENT( gI_text_vadj( current->text ) )->page_increment;
+	adjval = (ln * GTK_ADJUSTMENT( gI_text_vadj(current ) )->upper ) / tl - GTK_ADJUSTMENT( gI_text_vadj( current ) )->page_increment;
 
-	gtk_adjustment_set_value( GTK_ADJUSTMENT( gI_text_vadj( current->text )  ), adjval );
+	gtk_adjustment_set_value( GTK_ADJUSTMENT( gI_text_vadj( current )  ), adjval );
 
 
 	if( current->changed )
@@ -1227,22 +1300,22 @@ void goto_point( gint line )
 	point = get_point_from_line( line );
 
 	i = point;
-	gtk_editable_insert_text( GTK_EDITABLE( current->text ), " ", 1, &i );
-	gtk_editable_delete_text( GTK_EDITABLE( current->text ), i-1, i );
-	gI_text_set_point( current->text, i-2 );
+	gtk_editable_insert_text( GTK_EDITABLE( current ), " ", 1, &i );
+	gtk_editable_delete_text( GTK_EDITABLE( current ), i-1, i );
+	gI_text_set_point( current, i-2 );
 
 	if( changed == FALSE )
 	{
-		gI_document_set_unchanged( current );
-		gI_window_set_statusbar( current );
+		gI_document_set_changed_state( current, FALSE );
+		gI_window_set_statusbar( main_window );
 	}
 
 	/* select whole line */
-	edit_select_line( NULL, NULL );
+	edit_select_line( NULL, main_window );
 
 	/* track movement and set statusbar */
 	gI_document_track_movement( current );
-	gI_window_set_statusbar( current );
+	gI_window_set_statusbar( main_window );
 }
 
 
@@ -1253,41 +1326,17 @@ void goto_point( gint line )
  */
 void goto_line( gint line )
 {
-	gI_document *current;
+	GideDocument *current;
 	/*gint point;
 	int i;
 	glong changed;*/
 	double tl, ln, adjval;
 
-	current = gI_document_get_current( main_window );
+	current = gI_window_get_current_doc( main_window );
 	if( !current )
 		return;
 
-	/*** this is shit, replaced
-	    if( current->changed )
-	        changed = TRUE;
-	    else
-	        changed = FALSE;
-
-	    point = get_point_from_line( line );
-
-	    gI_text_set_point( current->text, point );
-
-	    i = point;
-	    gtk_editable_insert_text( GTK_EDITABLE( current->text ), " ", 1, &i );
-	    gtk_editable_delete_text( GTK_EDITABLE( current->text ), i-1, i );
-	    gI_text_set_point( current->text, i-2 );
-
-	    edit_select_line( NULL, NULL );
-	    
-	    if( changed == FALSE )
-	    {
-	        gI_document_set_unchanged( current );
-	   		gI_window_set_statusbar( current ); 
-	    }
-	***/  
-
-	tl = (double) get_total_lines( current->text );
+	tl = (double) get_total_lines( GTK_WIDGET(current) );
 	ln = (double) line;
 
 	if( tl < 10.0 )
@@ -1302,19 +1351,19 @@ void goto_line( gint line )
 		return;
 	}
 
-	gI_text_set_point( current->text, get_point_from_line( line ) );
+	gI_text_set_point( current, get_point_from_line( line ) );
 
 	/* borrowed from gEdit */
-	adjval = (ln * GTK_ADJUSTMENT( gI_text_vadj( current->text ) )->upper ) / tl - GTK_ADJUSTMENT( gI_text_vadj( current->text ) )->page_increment;
+	adjval = (ln * GTK_ADJUSTMENT( gI_text_vadj( current ) )->upper ) / tl - GTK_ADJUSTMENT( gI_text_vadj( current) )->page_increment;
 
-	gtk_adjustment_set_value( GTK_ADJUSTMENT( gI_text_vadj( current->text ) ), adjval );
+	gtk_adjustment_set_value( GTK_ADJUSTMENT( gI_text_vadj( current ) ), adjval );
 
 	/* select whole line */
-	edit_select_line( NULL, NULL );
+	edit_select_line( NULL, main_window );
 
 	/* track movement and set statusbar */
 	gI_document_track_movement( current );
-	gI_window_set_statusbar( current );
+	gI_window_set_statusbar( main_window );
 }
 
 
@@ -1406,7 +1455,9 @@ void jump_to_source( gchar *listitem )
 }
 
 
-void compile_list_select( GtkWidget *widget, gint row, gint column, GdkEventButton *bevent, gI_document *document )
+void
+compile_list_select( GtkWidget *widget, gint row, gint column,
+		     GdkEventButton *bevent, GideDocument *document )
 {
 	gchar *item_data_string;
 
@@ -1449,7 +1500,8 @@ glong get_total_lines( GtkWidget *text )
 }
 
 
-gchar *build_comp_string( gI_document *document, gchar *targetfile, gchar *fifoname, glong type )
+gchar *build_comp_string( GideDocument *document, gchar *targetfile,
+			  gchar *fifoname, glong type )
 {
 	static gchar cmdstr[MAXLEN];
 	gI_comp_set *cset;
@@ -1736,7 +1788,8 @@ gchar *match_cset_fpat( gI_comp_set *cset, gchar *filename )
 }
 
 
-gchar *parse_cmdline( gchar *cmdline, gI_comp_set *cset, gI_document *document, gchar *targetfile, glong type )
+gchar *parse_cmdline( gchar *cmdline, gI_comp_set *cset,
+		      GideDocument *document, gchar *targetfile, glong type )
 {
 	static gchar cmdstr[MAXLEN] = "";
 	gchar *ptr;
@@ -1940,7 +1993,7 @@ gI_comp_set *get_comp_set_from_filename( gchar *filename )
 
 void compile_syntax_check( GtkWidget *widget, gpointer data )
 {
-	gI_document *current = gI_document_get_current( main_window );
+	GideDocument *current = gI_window_get_current_doc( main_window );
 	gI_project *project;
 	gchar exec_string[MAXLEN];
 	gchar filename[STRLEN];
