@@ -1,4 +1,4 @@
-/* $Header: /cvs/gnome/gIDE/src/gI_file.c,v 1.5 1999/12/05 07:17:51 jpr Exp $ */
+/* $Header: /cvs/gnome/gIDE/src/gI_file.c,v 1.16 2000/04/16 04:59:56 jpr Exp $ */
 /* gIDE
  * Copyright (C) 1998-2000 Steffen Kern
  *
@@ -18,7 +18,7 @@
  */
 
 #include <config.h>
-#include <gnome.h>
+#include "gide.h"
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
@@ -26,11 +26,7 @@
 #include <stdlib.h>
 #include <sys/stat.h>
 
-#include "structs.h"
-#include "gide.h"
-#include "gI_document.h"
 #include "gI_file.h"
-#include "gI_window.h"
 #include "gI_compile.h"
 #include "gI_common.h"
 #include "gI_edit.h"
@@ -45,19 +41,13 @@
 
 
 /* globals */
-static glong noc;
 static GtkWidget *print_window = NULL;
 static GtkWidget *e_print_other;
 static glong print_lpr = TRUE;
-static glong file_exit_save = FALSE;
-static glong file_close_save = FALSE;
 static GtkWidget *r_print_all;
 static GtkWidget *r_print_sel;
-glong   ro_not_change = 0;
 
 /* externs */
-extern gI_window *main_window;
-extern gI_config *cfg;
 extern gchar gide_path[];
 
 #ifdef HAVE_GTKTEXT_PATCH
@@ -66,6 +56,11 @@ void            _gtk_editor_destroy_patterns   (GtkEditorHilitePatterns *pattern
 void            _gtk_editor_destroy_stable (GtkEditorSyntaxTable *stable);
 #endif
 
+/* Prototypes */
+static void file_save_by_name( GideWindow *window, GideDocument *document,
+			       gchar *filename );
+static gint file_close_dialog( GideWindow *window, GideDocument *current );
+static gboolean file_exit_is_changed( GideWindow *window );
 
 /*
  ---------------------------------------------------------------------
@@ -75,24 +70,28 @@ void            _gtk_editor_destroy_stable (GtkEditorSyntaxTable *stable);
 */
 gchar *file_strip_name( gchar *filename )
 {
-	gchar *striped = filename;
-	gint i;
+	return g_basename(filename);
+}
 
-    	if( !filename )
-        	return( NULL );
-    	if( strlen( filename ) == 0 )
-        	return( NULL );
-    
-    	for(i=strlen(filename);i>=0;i--)
-	{
-		if( filename[i] == '/' )
-        	{
-            		striped = &filename[i+1];
-            		break;
-        	}
+static void
+file_launch_external ( gchar *filename )
+{
+    	gchar launch[STRLEN];
+	gchar *ptr;
+	
+	strcpy( launch, cfg->redit );
+	ptr = strstr( launch, "%s" );
+	if( ptr ) {
+		*ptr = '\0';
+		if ( filename ) {
+			strcat( launch, filename );
+			ptr = strstr( cfg->redit, "%s" );
+			ptr++;ptr++;
+			strcat( launch, ptr );
+		}
 	}
-
-	return( striped );
+	strcat( launch, " &" );
+	system( launch );
 }
 
 
@@ -102,29 +101,19 @@ gchar *file_strip_name( gchar *filename )
      Desc: Callback-Function /File/New
  ---------------------------------------------------------------------
 */
-void file_new( GtkWidget *widget, gpointer data )
+void
+file_new( GtkWidget *widget, gpointer data )
 {
-	gchar HelpString[MAXLEN];
-    	gchar *ptr;
+	GideDocument *document;
     
-    	if( cfg->use_redit )
-    	{
-        	strcpy( HelpString, cfg->redit );
-        	ptr = strstr( HelpString, "%s" );
-        	if( ptr )
-        	{
-            		*ptr = ' ';
-            		ptr++;
-            		*ptr = ' ';
-        	}
+	g_assert(IS_GIDE_WINDOW(data));
 
-        	sprintf( HelpString, "%s &", HelpString );
-        	system( HelpString );
-    	}
-    	else
-    	{
-        	gI_document_new( main_window );
-    	}
+	if( cfg->use_redit ) {
+		file_launch_external( NULL );
+	} else {
+		document = GIDE_DOCUMENT( gI_document_new() );
+		gI_window_add_doc( GIDE_WINDOW(data), document );
+	}
 }
 
 
@@ -134,15 +123,17 @@ void file_new( GtkWidget *widget, gpointer data )
      Desc: Callback-Function /File/Open
  ---------------------------------------------------------------------
 */
-void file_open( GtkWidget *widget, gpointer data )
+void
+file_open( GtkWidget *widget, gpointer data )
 {
 	GList *files = gI_file_sel_new( _("Open File..."), FALSE, TRUE );
 
-	while( files )
-	{
+	g_assert(IS_GIDE_WINDOW(data));
+
+	while( files ) {
 		gchar *filename = (gchar *) files->data;
 
-    		file_open_by_name( main_window, filename );
+		file_open_by_name( GIDE_WINDOW(data), filename );
 
 		files = g_list_next( files );
 	}
@@ -151,217 +142,91 @@ void file_open( GtkWidget *widget, gpointer data )
 }
 
 
-void file_open_by_name( gI_window *window, gchar *filename )
+void
+file_open_by_name( GideWindow *window, gchar *filename )
 {
     	FILE *file;
     	gchar buf[STRLEN];
-    	gI_document *document;
-    	gchar HelpString[STRLEN];
-    	gchar *ptr;
+    	GideDocument *document;
 
     	/* only open the file, if its not already opened */
-    	if( check_file_open( filename ) )
-    	{
-		goto_file( filename );
+	if ( gI_window_is_open_doc(window, filename)) {
+		gI_window_goto_doc_by_file(window, filename);
+		return;
+	}
+	
+    	/* If the user chose an external editor */
+    	if( cfg->use_redit ) {
+		file_launch_external( filename );
 		return;
     	}
-
-    	/* bullshit below, better ignore it */
-    	if( cfg->use_redit )
-    	{
-        	strcpy( HelpString, cfg->redit );
-        	ptr = strstr( HelpString, "%s" );
-        	if( ptr )
-        	{
-            		*ptr = '\0';
-            		strcat( HelpString, filename );
-            		ptr = strstr( cfg->redit, "%s" );
-            		ptr++;ptr++;
-            		strcat( HelpString, ptr );
-        	}
-       		strcat( HelpString, " &" );
-        	system( HelpString );
-        	return;
-    	}
-    
-    	file = fopen( filename, "r" );
-    	if( file == NULL )
-    	{
+	
+	file = fopen( filename, "r" );
+    	if ( !file ) {
         	g_snprintf( buf, STRLEN, _("Unable to Open '%s'"), filename );
         	gI_error_dialog( buf );
         	return;
     	}
 
-
-    	document = gI_document_get_current( window );
-    	if( document )
-    	{
-        	if( document->filename != NULL || document->changed )
-        	{
-            		document = gI_document_new( window );
-        	}
-    	}
-    	else
-    	{
-		document = gI_document_new( window );
-    	}
-   
-    	gI_text_freeze( document->text );
-
-    	while( fgets( buf, sizeof(buf), file ) )
-    	{
-        	gI_text_insert( document->text, NULL, NULL, NULL,
-                         	 buf, strlen( buf ) );
-    	}
-
-    	gI_text_thaw( document->text );
-
-    	fclose( file );
-
-#ifdef HAVE_GTKTEXT_PATCH
-	if( cfg->highlight )
-      	{
-		gI_HilitePattern *pat = gI_glob_lookup( g_basename (filename) );
-		if (pat)
-	  	{
-	    		/* If it fails...we just return to default pattern... 
-	     		 * so there's no reson to test for return value here. */
-	    		gI_hilite_install_pattern( GTK_EDITOR( document->text ),
-				      pat );
-	  	}
-		gtk_editor_hilite_buffer( GTK_EDITOR( document->text ) );
-      	}
-    	else
-      	{
-		gtk_editor_hilite_when_idle( GTK_EDITOR( document->text ), FALSE );
+	document = gI_window_get_current_doc( window );
+    	if ( !document || !gI_document_is_free(document) ) {
+		document = GIDE_DOCUMENT( gI_document_new() );
+		gI_window_add_doc( window, document );
+		gI_document_load_file( document, filename, file );
+    	} else {
+		gI_document_load_file(document, filename, file);
 	}
-#endif
-
-    	document->filename = (gchar *) realloc( document->filename, (strlen( filename ) + 1) * sizeof( gchar ) );
-    	strcpy( document->filename, filename );
-    	if( strstr( filename, "/" ) )
-	{
-		gtk_label_set_text( GTK_LABEL( document->label ), file_strip_name( filename ) );
-	}
-	else
-	{
-		gtk_label_set_text( GTK_LABEL( document->label ), filename );
-	}
-
-	document->last_mod = get_last_mod( filename );
-
-    	/*gI_document_set_unchanged( document );*/
-
-    	if( !get_write_stat( document->filename ) )
-    	{
-        	document->read_only = 1;
-
-        	/* disconnect signal from menu item */
-
-        	/* gtk_signal_disconnect_by_func( GTK_OBJECT( menus_get_item_widget( "/Edit/Read Only" ) ), GTK_SIGNAL_FUNC( edit_read_only ), NULL ); */
-
-        	gI_window_set_visible_read_only( TRUE );
-
-        	ro_not_change = 1;
-
-        	/* set check menu item state & correct editable flag in widget */
-        	gtk_check_menu_item_set_state( GTK_CHECK_MENU_ITEM( menus_get_item_widget( "/Edit/Read Only" ) ), document->read_only );
-        	if( document->read_only )
-		{
-			gI_text_set_editable( document->text, FALSE );
-		}
-		else
-		{
-			gI_text_set_editable( document->text, TRUE );
-		}
-        	ro_not_change = 0;
-
-        	/* gtk_signal_connect( GTK_OBJECT( menus_get_item_widget( "/Edit/Read Only" ) ), "activate", GTK_SIGNAL_FUNC( edit_read_only ), NULL ); */
-    	} 
-
-    	gI_window_set_statusbar( document );
 }
 
-
-void file_open_cancel( GtkWidget *widget, GtkFileSelection *sel )
-{
-	if( GTK_WIDGET_VISIBLE( sel ) )
-        	gtk_widget_hide( GTK_WIDGET( sel ) );
-}
-
-
 /*
  ---------------------------------------------------------------------
      Function: file_reload()
-     Desc: Callback-Function /File/Reload
+     Desc: Callback-Function /File/Revert
  ---------------------------------------------------------------------
 */
-void file_reload( GtkWidget *widget, gpointer data )
+void
+file_reload( GtkWidget *widget, gpointer data )
 {
-	gI_document *current;
+	GideDocument *current;
+	gchar *filename;
 	FILE *file;
 	gchar buf[STRLEN];
 	gint pos;
 
-	current = gI_document_get_current( main_window );
+	g_assert(IS_GIDE_WINDOW(data));
+
+	current = gI_window_get_current_doc( GIDE_WINDOW(data) );
+	
 	if( !current )
+		return;
+
+    	if( !current->filename )
+        	return;
+
+	if ( !gI_document_is_changed(current) )
+		return;
+		
+	if( gI_ask_dialog( _("The file has been changed,\nDo You want to reload it?")) != 0 )  /* 0 == GNOME_YES */
 	{
 		return;
 	}
 
-    	if( current->filename == NULL )
-        	return;
-    
-    	if( current->changed == TRUE && (gint) data != 1 )
-    	{
-		if( gI_ask_dialog( _("The file has been changed,\nDo You want to reload it?")) != 0 )  /* 0 == GNOME_YES */
-		{
-	        	return;
-		}
-    	}
-
-    	pos = gI_text_get_point( current->text );
-    
-    	gI_document_clear_text( current );
-
+	pos = gI_text_get_point( current );
+	
     	file = fopen( current->filename, "r" );
-    	if( file == NULL )
-    	{
-        	g_snprintf( buf, STRLEN, _("Unable to Open '%s'"), current->filename );
+    	if( !file ) {
+        	g_snprintf( buf, STRLEN,
+			    _("Unable to Open '%s'"), current->filename );
         	gI_error_dialog( buf);
         	return; 
     	}
-    
-        gI_text_freeze( current->text );
 
-    	while( fgets( buf, 1024, file ) )
-	{
-		gI_text_insert( current->text, NULL, NULL, NULL,
-                         buf, strlen( buf ) );
-	}
+	filename = g_strdup( current->filename );
+	gI_document_load_file(current, filename, file);
+	g_free( filename );
 
-	fclose( file );
-
-#ifdef HAVE_GTKTEXT_PATCH
-    	if( cfg->highlight )
-    	{
-        	gtk_editor_hilite_buffer( GTK_EDITOR( current->text ) );
-   	}
-#endif
-
-    	gI_text_thaw( current->text );
-
-    	if( gI_text_get_length( current->text ) >= pos )
-        	gI_text_set_point( current->text, pos );
-
-    	gI_document_set_unchanged( current );
-
-    	gI_window_set_statusbar( current );
-
-    	gtk_widget_grab_focus( current->text );
-
-	/* update last mod time */
-	current->last_mod = get_last_mod( current->filename );
+    	if( gI_text_get_length( current ) >= pos )
+        	gI_text_set_point( current, pos );
 }
 
 
@@ -371,66 +236,35 @@ void file_reload( GtkWidget *widget, gpointer data )
      Desc: Callback-Function /File/Save
  ---------------------------------------------------------------------
 */
-void file_save( GtkWidget *widget, gpointer data )
+void
+file_save( GtkWidget *widget, gpointer data )
 {
 	FILE *file;
-	gI_document *current;
-    	glong length,i,current_no;
-	gchar *c;
+	GideWindow *window;
+	GideDocument *document;
 
-    	current = gI_document_get_current( main_window );
-	if( !current )
-	{
+	g_assert(IS_GIDE_WINDOW(data));
+
+	window = GIDE_WINDOW(data);
+	
+	document = gI_window_get_current_doc( window );
+	if( !document )
 		return;
-	}
 
-    	if( current->filename == NULL )
-    	{
+    	if( !document->filename ) {
         	file_save_as( widget, data );
         	return;
     	}
     
-    	file = fopen( current->filename, "w" );
-    	if( file == NULL )
-    	{
+    	file = fopen( document->filename, "w" );
+    	if( !file ) {
         	gI_error_dialog( _("Unable to Save File") );
         	return;
     	}
 
-    	length = gI_text_get_length( current->text );
-    	for(i=1;i<=length;i++)
-	{
-        	c = gtk_editable_get_chars( GTK_EDITABLE( current->text ), i-1, i );
-        	fputc( c[0], file );
-		g_free( c );
-	}
-
-    	fclose( file );
-
-	current->last_mod = get_last_mod( current->filename );
-
-    	gI_document_set_unchanged( current );
-
-    	gI_window_set_statusbar( current );
-
-    	if( file_exit_save )
-    	{
-        	if( gI_document_get_no( main_window )-1 == 0 )
-            		gtk_main_quit();
-    	}
-    	if( file_close_save )
-    	{
-        	current_no = gtk_notebook_current_page( GTK_NOTEBOOK( main_window->notebook ) );
-
-        	gtk_notebook_remove_page( GTK_NOTEBOOK( main_window->notebook ), current_no );
-        	g_list_remove( main_window->documents, current );
-
-    		free_op_list( current->op_list );
-
-		/* free memory */
-		g_free( current->filename );
-		g_free( current );	
-    	}
+	gI_document_save_file( document, document->filename, file );
+	
+    	gI_window_set_statusbar( window );
 }
 
 
@@ -442,261 +276,78 @@ void file_save( GtkWidget *widget, gpointer data )
 */
 void file_save_as( GtkWidget *widget, gpointer data )
 {
-	GList *files = gI_file_sel_new( _("Save As..."), TRUE, FALSE );
-	gI_document *current = gI_document_get_current( main_window );
+	GideWindow *window;
+	GideDocument *document;
+	GList *files;
 
-	if( files != NULL )
-	{
+	g_assert(IS_GIDE_WINDOW(data));
+
+	window = GIDE_WINDOW( data );
+	document = gI_window_get_current_doc( window );	
+	
+	files = gI_file_sel_new( _("Save As..."), TRUE, FALSE );
+
+	if( files ) {
 		gchar *filename = files->data;
-		if( !filename ) 
-		{
+		if( !filename )
 			return;
-		}
 
-        	current->filename = (gchar *) realloc( current->filename, (strlen( filename ) + 1) * sizeof( gchar ) );
-		strcpy( current->filename, filename );
-	        file_save_by_name( main_window, filename );
-	}
-	else
-	{
+	        file_save_by_name( window, document, filename );
+	} else {
 		return;
 	}	
 
 	gI_file_sel_free_list( files );
-
-	if( file_exit_save )
-	{
-		if( gI_document_get_no( main_window ) == 0 )
-			gtk_main_quit();
-	}
-
-	if( file_close_save )
-	{
-		glong current_no = gtk_notebook_get_current_page( GTK_NOTEBOOK( main_window->notebook ) );
-
-		gtk_notebook_remove_page( GTK_NOTEBOOK( main_window->notebook ), current_no );
-		g_list_remove( main_window->documents, current );
-
-		free_op_list( current->op_list );
-
-		/* free memory */
-		g_free( current->filename );
-		g_free( current );	
-	}
 }
 
-
-static void file_save_overwrite( GtkWidget *widget, GtkWidget *dialog )
-{
-	gI_document *current;
-	gchar *filename;
-	glong length, i;
-	FILE *file;
-	gchar *c;
-
-	if( dialog )
-	{
-		gtk_widget_destroy( dialog );
-	}
-
-	current = gI_document_get_current( main_window );
-	g_return_if_fail( current != NULL );
-
-	filename = current->filename;
-
-	file = fopen( filename, "w" );
-	if( file == NULL )
-	{
-		gI_error_dialog( _("Unable to Save File") );
-		return;
-    	}
-    
-    	length = gI_text_get_length( current->text );
-
-    	for(i=1;i<=length;i++)
-	{
-        	c = gtk_editable_get_chars( GTK_EDITABLE( current->text ), i-1, i );
-        	fputc( c[0], file );
-		g_free( c );
-	}
-
-    	fclose( file );
-
-	current->last_mod = get_last_mod( current->filename );
-
-    	/*
-	    current->changed = FALSE;
-	    current->changed_id = gtk_signal_connect( GTK_OBJECT( current->text ), "changed",
-                                              GTK_SIGNAL_FUNC( gI_document_changed ), current );
-	    current->label = gtk_label_new( current->filename );
-	*/
-
-    	gI_document_set_unchanged( current );
-
-    	gI_window_set_statusbar( current );
-}
-
-
-static void file_save_abort( GtkWidget *widget, GtkWidget *dialog )
-{
-	if( dialog )
-	{
-		gtk_widget_destroy( dialog );
-
-	}
-}
-
-
-void file_save_by_name( gI_window *window, gchar *filename )
+static void
+file_save_by_name( GideWindow *window, GideDocument *document,
+		   gchar *filename )
 {
 	FILE *file;
-	gI_document *current;
-	glong length,i;
 	gchar warnmsg[STRLEN];
-	gchar *c;
 	gint btn;
 
-	if( file_exist( filename ) != 0 )
-	{
+	if( file_exist( filename ) != 0 ) {
 		/* overwrite dialog */
-		sprintf( warnmsg, _("The File %s already exists, do you really want to overwrite it?"), filename );
+		sprintf( warnmsg,
+			 _("The File %s already exists. Overwrite it?"),
+			 filename );
 	        btn = gI_ask_dialog( warnmsg);
         
-        	switch (btn)
-		{
+        	switch (btn) {
 			/* YES */
 			case 0:
-		    	{
-				file_save_overwrite( NULL, NULL );
 			    	break;
-			}
 
-			/* NO */
+			/* NO, CANCEL or CLOSE */
 	    		case 1:
-			{
-				file_save_abort( NULL, NULL );
-		    		break;
-			}	
-
-			/* CANCEL/CLOSE */
 	    		case 2:
 	    		case -1:
-			{
+				return;
 				break;
-			}
 		}
-	
-		return;
 	}
 
 	file = fopen( filename, "w" );
-	if( file == NULL )
-    	{
+	if( !file ) {
         	gI_error_dialog( _("Unable to Save File") );
         	return;
     	}
-    
-    	current = gI_document_get_current( window );
-	g_return_if_fail( current != NULL );	
 
-	length = gI_text_get_length( current->text );
-    	for(i=1;i<=length;i++)
-	{
-        	c = gtk_editable_get_chars( GTK_EDITABLE( current->text ), i-1, i );
-	        fputc( c[0], file );
-		g_free( c );
-	}
-
-	fclose( file );
-
-	/* correct last modification time after save */
-	current->last_mod = get_last_mod( current->filename );	
-
-	/*
-	current->changed = FALSE;
-	current->changed_id = gtk_signal_connect( GTK_OBJECT( current->text ), "changed",
-                                              GTK_SIGNAL_FUNC( gI_document_changed ), current );
-	current->label = gtk_label_new( current->filename );
-	*/
-
-#ifdef HAVE_GTKTEXT_PATCH
-	if( cfg->highlight )
-        {
-	        gI_HilitePattern *pat = gI_glob_lookup( g_basename (filename) );
-                if (pat)
-	        {
-		        /* If it fails...we just return to default pattern...
-	 		 * so there's no reson to test for return value here. */
-                        gI_hilite_install_pattern( GTK_EDITOR( current->text ), pat );
-		}
-		else
-		{
-			/* syntaxtable.h says, that i should not use this functions, so
-			 * is there another legal way to just remove the syntax table and the
-			 * patterns ??? */
-			_gtk_editor_destroy_stable( GTK_EDITOR( current->text )->stable );
-			_gtk_editor_destroy_patterns( GTK_EDITOR( current->text )->patterns );
-			GTK_EDITOR( current->text )->stable = NULL;
-			GTK_EDITOR( current->text )->patterns = NULL;
-		}
-
-		gtk_editor_hilite_buffer( GTK_EDITOR( current->text ) );
-       }
-       else
-       {
-	       gtk_editor_hilite_when_idle( GTK_EDITOR( current->text ), FALSE );
-       }
-#endif
-
-    	gI_document_set_unchanged( current );
-
-    	gI_window_set_statusbar( current );
+	gI_document_save_file( document, filename, file );
 }
 
-
-void file_save_cancel( GtkWidget *widget, GtkFileSelection *sel )
-{
-	glong current_no;
-	gI_document *current;
-
-	current = gI_document_get_current( main_window );
-	g_return_if_fail( current != NULL );
-
-	if( GTK_WIDGET_VISIBLE( sel ) )
-	{
-		gtk_widget_hide( GTK_WIDGET( sel ) );
-	}
-
-	if( file_exit_save )
-	{
-		if( gI_document_get_no( main_window ) == 0 )
-            		gtk_main_quit();
-    	}
-
-	if( file_close_save )
-	{
-        	current_no = gtk_notebook_get_current_page( GTK_NOTEBOOK( main_window->notebook ) );
-
-        	gtk_notebook_remove_page( GTK_NOTEBOOK( main_window->notebook ), current_no );
-        	g_list_remove( main_window->documents, current );
-
-    		free_op_list( current->op_list );
-
-		/* free memory */
-		g_free( current->filename );
-		g_free( current );	
-    	}
-}
-
-
-static void print_destroy( GtkWidget *widget, gpointer data )
+static void
+print_destroy( GtkWidget *widget, gpointer data )
 {
 	gtk_widget_destroy( print_window );
 	print_window = NULL;
 }
 
 
-static void print_show_lpr( GtkWidget *widget, gpointer data )
+static void
+print_show_lpr( GtkWidget *widget, gpointer data )
 {
 	gtk_widget_set_sensitive( e_print_other, FALSE );
 	print_lpr = TRUE;
@@ -713,7 +364,7 @@ static void print_show_other( GtkWidget *widget, gpointer data )
 static void _file_print( GtkWidget *widget, gpointer data )
 {
 	FILE *file;
-	gI_document *current;
+	GideDocument *current;
 	glong length,i;
 	gchar filename[MAXLEN];
 	gchar tmpstring[MAXLEN];
@@ -722,7 +373,7 @@ static void _file_print( GtkWidget *widget, gpointer data )
 	gchar *var_ptr;
 	gchar *c;
 
-	current = gI_document_get_current( main_window );
+	current = gI_window_get_current_doc( main_window );
 	g_return_if_fail( current != NULL );
 
 	if( print_lpr )
@@ -749,17 +400,17 @@ static void _file_print( GtkWidget *widget, gpointer data )
 
             		if( GTK_TOGGLE_BUTTON( r_print_sel )->active )
             		{
-                		if( !GTK_EDITABLE( current->text )->has_selection )
+                		if( !GTK_EDITABLE( current )->has_selection )
                 		{
                     			gI_error_dialog( _("No selection in current document!") );
                     			return;
                 		}
                 
-                		for(i=GTK_EDITABLE( current->text )->selection_start_pos+1;
-                    		    i<=GTK_EDITABLE( current->text )->selection_end_pos;
+                		for(i=GTK_EDITABLE( current )->selection_start_pos+1;
+                    		    i<=GTK_EDITABLE( current )->selection_end_pos;
                     		    i++)
 				{
-                    			c = gtk_editable_get_chars( GTK_EDITABLE( current->text ), i-1, i );
+                    			c = gtk_editable_get_chars( GTK_EDITABLE( current ), i-1, i );
 					fputc( c[0], file );
 					g_free( c );
 				}
@@ -767,11 +418,11 @@ static void _file_print( GtkWidget *widget, gpointer data )
 
             		if( GTK_TOGGLE_BUTTON( r_print_all )->active )
             		{
-                		length = gI_text_get_length( current->text );
+                		length = gI_text_get_length( current );
 
     				for(i=1;i<=length;i++)
 				{
-			        	c = gtk_editable_get_chars( GTK_EDITABLE( current->text ), i-1, i );
+			        	c = gtk_editable_get_chars( GTK_EDITABLE( current ), i-1, i );
 			        	fputc( c[0], file );
 					g_free( c );
 				}
@@ -837,11 +488,11 @@ static void _file_print( GtkWidget *widget, gpointer data )
 
             		if( GTK_TOGGLE_BUTTON( r_print_all )->active )
             		{
-                		length = gI_text_get_length( current->text );
+                		length = gI_text_get_length( current );
 
     				for(i=1;i<=length;i++)
 				{
-			        	c = gtk_editable_get_chars( GTK_EDITABLE( current->text ), i-1, i );
+			        	c = gtk_editable_get_chars( GTK_EDITABLE( current ), i-1, i );
 			        	fputc( c[0], file );
 					g_free( c );
 				}
@@ -995,96 +646,69 @@ void file_print( GtkWidget *widget, gpointer data )
      Desc: Callback-Function /File/Close
  ---------------------------------------------------------------------
 */
-void file_close( GtkWidget *widget, gpointer data)
+void
+file_close( GtkWidget *widget, gpointer data)
 {
-	gI_document *current;
-	gint current_no;
+	GideWindow *window;
+	GideDocument *document;
 
-	current = gI_document_get_current( main_window );
-	if( !current )
-	{
+	g_assert(IS_GIDE_WINDOW(data));
+	window = GIDE_WINDOW(data);
+	document = gI_window_get_current_doc( window );
+
+	if( !document )
 		return;
-	}
 
-    	if( current->changed )
-    	{
-		if( !file_close_changed_dialog_modal(current) )  /* Return false means dont close until */ 
-	        	return; /* save done or user press cancel */            
+    	if( gI_document_is_changed( document ) ) { 
+		if( file_close_dialog(window, document) == 2 )
+			return; 
     	}
-    
-    	current_no = gtk_notebook_get_current_page( GTK_NOTEBOOK( main_window->notebook ) );
 
-    	gtk_notebook_remove_page( GTK_NOTEBOOK( main_window->notebook ), current_no );
-    	free_op_list( current->op_list );
-
-    	g_list_remove( main_window->documents, current );
-
-    	/* free memory */
-    	g_free( current->filename );
-   	g_free( current );	
-
-    	if( gI_document_get_no( main_window ) == 0 )
-    	{
-        	gtk_window_set_title( GTK_WINDOW( main_window->window ), "gIDE " VERSION );
-        	gI_window_clear_statusbar( main_window );
-		gI_window_set_visible_read_only( FALSE ); 
-    	}
+	gtk_object_destroy( GTK_OBJECT(document) );
 }
 
 
 /* -------------------------------------*/
-/* file_close_changed_dialog_modal()                       */
+/* file_close_dialog()                       */
 /* return TRUE if user press NO and file is NOT saved      */
 /* Return FALSE if file will save or user press cancel button*/
 /* Use this instead of file_close_changed_dialog */
 /* --------------------------------------*/
-gint file_close_changed_dialog_modal(gI_document *current)
+static gint
+file_close_dialog( GideWindow *window, GideDocument *current )
 {
+	gchar *filename;
 	gchar HelpString[STRLEN];
-	gint user_reply;
+	gint btn;
 
-	if( !current->filename )
-	{
-		gchar *label, *label_dup;
-		gchar *ptr;
+	if( !current->filename ) {
+		gchar *label;
 
-		gtk_label_get( GTK_LABEL( current->label ), &label );
-
-		label_dup = g_strdup( label );
-		label = label_dup;
-
-		ptr = strchr( label, '*' );
-		if( ptr )
-			*ptr = '\0';
-
-        	current->filename = g_strdup( label );
-
-		g_free( label_dup );
+		label = g_strdup( gI_window_get_doc_label(window, current) );
+		
+        	filename = g_strdup(file_strip_name( label ));
+	} else {
+		filename = g_strdup( current->filename ); 
 	}
 
-	g_snprintf( HelpString, STRLEN, _("\nThe File %s has been changed!\nDo You want to save it?\n"), file_strip_name( current->filename ) );
+	g_snprintf( HelpString, STRLEN,
+		    _("The file \"%s\" has been modified!\nSave it?\n"),
+		    filename );
+	g_free(filename);
 	
-	user_reply = gI_ask_dialog( HelpString );
-	if( user_reply == 2 )   /* CANCEL */
-		return FALSE;
-
-   	if( user_reply == 0 )   /* YES */
-   	{
-		file_close_save = TRUE;
-		if( current->filename == NULL )
-			file_save_as( NULL, NULL );
+	btn = gI_ask_dialog( HelpString );
+	if ( btn == 0 ) {
+		if( !current->filename )
+			file_save_as( NULL, window );
 		else
-			file_save( NULL, NULL );
-    	    
-		file_close_save = FALSE;
-		return( FALSE );
+			file_save( NULL, window );
 	}
 
-	return( TRUE );
+	return btn;
 }
 
 
-static void write_history( gI_document *document, FILE *history )
+static void write_history( GideDocument *document, FILE *history )
 {
 	if( !document )
         	return;
@@ -1104,183 +728,66 @@ static void write_history( gI_document *document, FILE *history )
 */
 void file_exit( GtkWidget *widget, gpointer data )
 {
-	gint nod,i,changed=0;
-	gI_document *current;
+	gint nod;
+	GideWindow *window;
 	FILE *history;
     	gchar historywp[MAXLEN];
     	gI_project *project;
 
-    	/* close opened project */
-    	if( (project = gI_project_get_current()) )
-    	{
-        	project_close( NULL, NULL );
-    	}
+	g_assert(IS_GIDE_WINDOW(data));
 
-    	/* save filenames */
-    	g_snprintf( historywp, MAXLEN, "%s/%s", gide_path, HISTORY );
-    	history = fopen( historywp, "w" );
-    	if( history )
-    	{
-        	g_list_foreach( main_window->documents, (GFunc) write_history, (gpointer) history );
-        	fclose( history );
-    	}
-
-    	nod = gI_document_get_no( main_window );
-
-    	for(i=1;i<=nod;i++)
-    	{
-        	gtk_notebook_set_page( GTK_NOTEBOOK( main_window->notebook ), i );
-		current = gI_document_get_current( main_window );
-        	if( current->changed == TRUE )
-        	{
-            		changed = TRUE;
-            		break;
-        	}
-    	}
-
-    	if( changed )
-	{
-		file_exit_is_changed( main_window );
+	window = GIDE_WINDOW(data);
+	
+	/* close opened project */
+	if( (project = gI_project_get_current()) ) {
+    	gI_project_close( NULL, NULL );
 	}
-	else
-	{
-		gtk_main_quit();
 
+	/* save filenames */
+	g_snprintf( historywp, MAXLEN, "%s/%s", gide_path, HISTORY );
+	history = fopen( historywp, "w" );
+	if( history ) {
+    	g_list_foreach( window->documents,
+			(GFunc) write_history,
+			(gpointer) history );
+    	fclose( history );
+	}
+
+	nod = gI_window_num_changed_docs( window );
+
+	if( nod > 0 ) {
+		if (!file_exit_is_changed( window ))
+			gtk_main_quit();
+	} else {
+		gtk_main_quit();
 	}
 }
 
-
-void file_exit_is_changed( gI_window *window )
+static gboolean
+file_exit_is_changed( GideWindow *window )
 {
+    	GideDocument *document;
+	gboolean canceled;
 	gint nod,i;
-    	gI_document *current;
 
-    	nod = gI_document_get_no( main_window );
-	noc = gI_document_get_changed( main_window ); 
-    
-    	for(i=0;i<nod;i++)
-    	{
-        	gtk_notebook_set_page( GTK_NOTEBOOK( window->notebook ), i );
-        	current = gI_document_get_current( window );
-        	if( current->changed == TRUE )
-        	{
-            		file_exit_changed_dialog( current );
+    	nod = gI_window_num_docs( main_window );
+	canceled = FALSE;
+	
+    	for( i=0; i<nod; i++ ) {
+        	gI_window_goto_doc_by_index( window, i );
+        	document = gI_window_get_current_doc( window );
+        	if( gI_document_is_changed(document) ) {
+            		if (file_close_dialog( window, document ) == 2) {
+				canceled = TRUE;
+			}
         	}
     	}
+
+	return canceled;
 }
 
-
-void file_exit_changed_dialog( gI_document *document )
-{
-/***
-    GtkWidget *dialog;
-    GtkWidget *vbox, *hbox, *separator, *label, *ok, *cancel;
-***/
-    	gchar HelpString[STRLEN];
-
-    	if( !document->filename )
-	{
-		gchar *label, *label_dup;
-		gchar *ptr;
-
-		gtk_label_get( GTK_LABEL( document->label ), &label );
-
-		label_dup = g_strdup( label );
-		label = label_dup;
-
-		ptr = strchr( label, '*' );
-		if( ptr )
-			*ptr = '\0';
-
-        	document->filename = g_strdup( label );
-
-		g_free( label_dup );
-	}
-
-    	g_snprintf( HelpString, STRLEN, _("\nThe File\n'%s'\nhas been changed!\nDo You want to save it?\n"), file_strip_name( document->filename ) );
-
-	if( gI_ask_dialog( HelpString ) == 0 )  /* YES */
-	{
-		file_exit_changed_ok( NULL, NULL );
-	}
-	else
-	{
-		file_exit_changed_cancel( NULL, NULL );
-	}
-
-/***
-    	dialog = gtk_window_new( GTK_WINDOW_DIALOG );
-    	gtk_widget_set_usize( dialog, 250, 175 );
-
-    	vbox = gtk_vbox_new( FALSE, 0 );
-    	gtk_container_add( GTK_CONTAINER( dialog ), vbox );
-    	gtk_widget_show( vbox );
-
-    	hbox = gtk_hbox_new( FALSE, 0 );
-    	gtk_box_pack_start( GTK_BOX( vbox ), hbox, FALSE, TRUE, 5 );
-    	gtk_widget_show( hbox );
-
-    	if( !document->filename )
-        	document->filename = g_strdup( "Untitled" );
-    
-
-    	g_snprintf( HelpString, STRLEN, "\nThe File %s has been changed!\nDo You want to save it?\n", file_strip_name( document->filename ) );
-    	label = gtk_label_new( HelpString );
-    	gtk_container_add( GTK_CONTAINER( hbox ), label );
-    	gtk_widget_show( label );
-    
-    	separator = gtk_hseparator_new();
-    	gtk_box_pack_start( GTK_BOX( vbox ), separator, FALSE, TRUE, 5 );
-    	gtk_widget_show( separator );
-
-    	hbox = gtk_hbox_new( FALSE, 0 );
-    	gtk_box_pack_start( GTK_BOX( vbox ), hbox, FALSE, TRUE, 5 );
-    	gtk_widget_show( hbox );
-
-    	ok = gtk_button_new_with_label( "Save" );
-    	gtk_box_pack_start( GTK_BOX( hbox ), ok, TRUE, TRUE, 15 );
-    	gtk_signal_connect( GTK_OBJECT( ok ), "clicked",
-                       	    GTK_SIGNAL_FUNC( file_exit_changed_ok ), document );
-    	gtk_widget_show( ok );
-    
-    	cancel = gtk_button_new_with_label( "Discard" );
-    	gtk_box_pack_start( GTK_BOX( hbox ), cancel, TRUE, TRUE, 15 );
-    	gtk_signal_connect( GTK_OBJECT( cancel ), "clicked",
-                            GTK_SIGNAL_FUNC( file_exit_changed_cancel ), document );
-    	gtk_widget_show( cancel );
-    
-    	gtk_widget_show( dialog );
-***/
-
-}
-
-
-void file_exit_changed_ok( GtkWidget *widget, gpointer data )
-{
-	/* save */
-	file_exit_save = TRUE;
-	file_save( NULL, NULL );
-	file_exit_save = FALSE;
-}
-
-
-void file_exit_changed_cancel( GtkWidget *widget, gpointer data )
-{
-	noc--;
-
-	if( noc < 0 )
-	{
-		g_warning( "NOC < 0: Something is going wrong (%s/%d)!\n", __FILE__, __LINE__ );
-   	}
-
-    	if( noc == 0 )
-	{
-		gtk_main_quit();
-	}
-}
-
-
-static void _file_autosave( gI_document *document, gpointer data )
+static void
+_file_autosave( GideDocument *document, gpointer data )
 {
 	FILE *docfile;
 	gchar str[STRLEN];
@@ -1307,21 +814,21 @@ static void _file_autosave( gI_document *document, gpointer data )
         	return;
     	}
 
-    	length = gI_text_get_length( document->text );
+    	length = gI_text_get_length( document );
     	for(i=1;i<=length;i++)
 	{
-        	c = gtk_editable_get_chars( GTK_EDITABLE( document->text ), i-1, i );
+        	c = gtk_editable_get_chars( GTK_EDITABLE( document ), i-1, i );
 		fputc( c[0], docfile );
 		g_free( c );
 	}
 
     	fclose( docfile );
 
-    	gI_document_set_unchanged( document );
+    	gI_document_set_changed_state( document, FALSE );
 
 	document->last_mod = get_last_mod( document->filename );
 
-	gI_window_set_statusbar( document );
+	gI_window_set_statusbar( main_window );
 }
 
 
@@ -1391,7 +898,7 @@ gint file_check_if_exist( gchar *fname, gint askcreate )
  * 	   1 - changed
  *	   2 - does not (longer) exist
  */
-glong check_doc_changed( gI_document *doc )
+glong check_doc_changed( GideDocument *doc )
 {
 	if( !doc )
 		return( 0 );
@@ -1409,3 +916,53 @@ glong check_doc_changed( gI_document *doc )
 }
 
 
+/*
+ * Close all open files
+ */
+void
+gI_file_close_all(GtkWidget *widget, gpointer data)
+{
+	gint								i;
+
+	for(i = 0; i < gI_window_num_docs(main_window); i++)
+	{
+		gtk_notebook_set_page(GTK_NOTEBOOK(main_window->notebook), i);
+		file_close(NULL, main_window);
+	}
+}
+
+
+/*
+ * Save all files
+ */
+void
+gI_file_save_all(GtkWidget *widget, gpointer data)
+{
+	gint i;
+	GideWindow *window;
+	gint nod;
+	GideDocument *document;
+
+	g_return_if_fail(IS_GIDE_WINDOW(data));
+
+	window = GIDE_WINDOW(data);
+
+	nod = gI_window_num_changed_docs(window);
+
+	if(nod > 0) {
+		for(i = 0; i < gI_window_num_docs(window); i++) {
+			gtk_notebook_set_page(GTK_NOTEBOOK(window->notebook), i);
+			document = gI_window_get_current_doc(window);
+			if (gI_document_is_changed(document)) {
+				file_save(NULL, window);
+			}
+		}
+	}
+}
+
+
+
+
+
+
+
