@@ -1,4 +1,4 @@
-/* $Header: /cvs/gnome/gIDE/src/gI_flist.h,v 1.2 1999/11/06 17:45:39 jpr Exp $ */
+/* $Header: /cvs/gnome/gIDE/src/gI_flist.h,v 1.3 2000/04/22 17:23:01 jpr Exp $ */
 /* gIDE
  * Copyright (C) 1998-2000 Steffen Kern
  *
@@ -26,7 +26,7 @@
  * Prototypes for 'gI_flist.c'
  */
 void gen_files_list( gchar *dirname, GtkWidget *window );
-GtkWidget *setup_list_window();
+GtkWidget *setup_list_window(void);
 void flist( void );
 
 
