@@ -1,4 +1,4 @@
-/* $Header: /cvs/gnome/gIDE/src/gI_ftree.h,v 1.2 1999/11/06 17:45:39 jpr Exp $ */
+/* $Header: /cvs/gnome/gIDE/src/gI_ftree.h,v 1.3 2000/03/21 17:42:17 jpr Exp $ */
 /* gIDE
  * Copyright (C) 1998-2000 Steffen Kern
  *
@@ -25,19 +25,7 @@
 /*
  * Prototypes for 'gI_ftree.c'
  */
-void gen_files_tree( gchar *dirname, GtkWidget *window );
-void setup_tree( GtkWidget *window );
-GtkWidget *setup_tree_window();
 void ftree(void);
 
-
-typedef struct _gI_ftree_item_info gI_ftree_item_info;
-struct _gI_ftree_item_info
-{
-    GtkWidget *window;
-    gchar *filename;
-    glong isdir;
-};
-
 #endif
 
