@@ -1,4 +1,4 @@
-/* $Header: /cvs/gnome/gIDE/src/gI_functions.h,v 1.2 1999/11/06 17:45:39 jpr Exp $ */
+/* $Header: /cvs/gnome/gIDE/src/gI_functions.h,v 1.4 2000/04/22 17:23:01 jpr Exp $ */
 /* gIDE
  * Copyright (C) 1998-2000 Steffen Kern
  *
@@ -24,20 +24,19 @@
  * Prototypes for 'gI_functions.c'
  */
 
-typedef struct _c_status c_status;
-struct _c_status
+typedef struct _c_status
 {
-    glong klammern_1;
-    glong klammern_2;
-    glong klammern_3;
-    glong comment;
-    glong hyph;
-    glong dhyph;
-};
+	glong curly_brackets;
+	glong round_brackets;
+	glong square_brackets;
+	glong comment;
+	glong hyph;
+	glong dhyph;
+} c_status;
 
-glong c_parse_special( gchar *buf, glong cc, c_status *c_status );
-glong init_cstatus( c_status *c_status );
-void show_all_functions();
+glong c_parse_special(gchar* buf, glong cc, c_status* c_status);
+glong init_cstatus(c_status* c_status);
+void show_all_functions(void);
 
 #endif
 
