@@ -1,4 +1,4 @@
-/* $Header: /cvs/gnome/gIDE/src/gI_help.c,v 1.3 1999/11/06 17:45:39 jpr Exp $ */
+/* $Header: /cvs/gnome/gIDE/src/gI_help.c,v 1.4 2000/03/21 17:42:17 jpr Exp $ */
 /* gIDE
  * Copyright (C) 1998-200-2000 Steffen Kern
  *
@@ -204,7 +204,7 @@ void help_man_page( GtkWidget *widget, gpointer data )
 }
 
 /* convert a selection into something usable by gman() */
-void man_page_click( GtkWidget *widget, gpointer this ) {
+static void man_page_click( GtkWidget *widget, gpointer this ) {
 	static GdkAtom targets_atom = GDK_NONE;
 	
 	if (targets_atom == GDK_NONE)
@@ -215,7 +215,7 @@ void man_page_click( GtkWidget *widget, gpointer this ) {
 }
 
 /* call gman() with data from man_page_click() then unselect selection */
-void man_sel_recv( GtkWidget *widget, GtkSelectionData *selection_data,
+static void man_sel_recv( GtkWidget *widget, GtkSelectionData *selection_data,
 					gpointer this ) {
 	if (selection_data->length <= 0)
 		return; 
@@ -226,7 +226,7 @@ void man_sel_recv( GtkWidget *widget, GtkSelectionData *selection_data,
 }
 
 /* Read in text from entry box, and then goto corresponding man page */
-void man_goto( GtkWidget *widget, GtkEntry *entry ) {
+static void man_goto( GtkWidget *widget, GtkEntry *entry ) {
 	gman(gtk_entry_get_text(entry), NULL);
 }
 
@@ -240,7 +240,7 @@ typedef struct {
 } text_entry_st;
 
 /* List releated man pages */
-void man_apro( GtkWidget *widget, text_entry_st *tes ) {
+static void man_apro( GtkWidget *widget, text_entry_st *tes ) {
 	char cmdstr[256], line[1024];
 	FILE *fp;
 
@@ -290,7 +290,7 @@ void man_apro( GtkWidget *widget, text_entry_st *tes ) {
 }
 
 /* list an index of man pages */
-void man_indx( GtkWidget *widget, text_entry_st *tes ) {
+static void man_indx( GtkWidget *widget, text_entry_st *tes ) {
 	char Section[10],temp[60];
 	int i = 0;
 
