@@ -1,4 +1,4 @@
-/* $Header: /cvs/gnome/gIDE/src/gI_hilite.c,v 1.5 1999/11/23 21:14:10 jpr Exp $ */
+/* $Header: /cvs/gnome/gIDE/src/gI_hilite.c,v 1.9 2000/04/16 04:59:56 jpr Exp $ */
 /*  gIDE
  *  Copyright (C) 1998-2000 Steffen Kern
  *
@@ -21,25 +21,19 @@
 
 #ifdef HAVE_GTKTEXT_PATCH
 
-#include <gnome.h>
+#include "gide.h"
 #include <gtkeditor/gtkeditor.h>
 
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
 
-#include "structs.h"
-#include "gI_document.h"
 #include "gI_menus.h"
 
 #include "gI_common.h"
 
 #include "gI_hilite.h"
 
-extern gI_window *main_window;
-extern gI_config *cfg;
-extern gchar gide_path[];
-
 typedef struct _Pattern {
   gchar                   *name;
   gI_HilitePattern  pat;
@@ -278,7 +272,7 @@ gI_hilite_get_pattern_widget (gI_pat_dialog *pd)
                    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 
   /* Call this now to show the patterns initially on display */
-  patname_changed_cb( GTK_COMBO(patname)->entry ,pd);
+  patname_changed_cb( GTK_ENTRY(GTK_COMBO(patname)->entry) ,pd);
 
   return vbox;
 }
@@ -292,10 +286,10 @@ gI_hilite_set_buffer_patterns_cb (void)
   GtkWidget *bbox;
   GtkWidget *ok, *cancel;
 
-  gI_document *doc = gI_document_get_current ( main_window );
+  GideDocument *doc = gI_window_get_current_doc ( main_window );
 
   gI_pat_dialog *pd = g_new (gI_pat_dialog, 1);
-  pd->buf = GTK_EDITOR ( doc->text );
+  pd->buf = GTK_EDITOR ( doc );
 
   dialog = gtk_dialog_new ();
   pd->dialog = dialog;
@@ -340,9 +334,9 @@ void
 gI_hilite_rehilite (void)
 {
   GtkEditor *buf;
-  gI_document *doc = gI_document_get_current ( main_window );
+  GideDocument *doc = gI_window_get_current_doc ( main_window );
   if( !doc ) return;
-  buf = GTK_EDITOR ( doc->text );
+  buf = GTK_EDITOR ( doc );
   gtk_editor_hilite_buffer (buf);
 }
 
@@ -718,7 +712,7 @@ gI_hilite_set_pattern (GtkWidget *buf, gchar *pattern_name)
 /* FIXME: The non-guile stuff is *very* primitive...they should probably be
  * be reimplemented or discarded all together if we move entirely to guile */
 
-#if !HAVE_LIBGUILE
+#if !HAVE_GUILE
 static gchar*
 check_string (FILE *file) 
 {
@@ -1103,7 +1097,7 @@ gI_hilite_load_patterns ( const gchar *filename )
 
   fclose (file);
 }
-#endif /*!HAVE_LIBGUILE*/
+#endif /*!HAVE_GUILE*/
 
 static void
 quotestr (const gchar *src, gchar *dst)
@@ -1125,7 +1119,7 @@ quotestr (const gchar *src, gchar *dst)
   dst[j] = '\0';
 }
 
-#if HAVE_LIBGUILE
+#if HAVE_GUILE
 static void
 save_pattern_map (Pattern *pat, FILE *file)
 {
@@ -1220,7 +1214,7 @@ gI_hilite_save_patterns ( const gchar *filename )
   fclose (file);
 }
 
-#else  /* if !HAVE_LIBGUILE */
+#else  /* if !HAVE_GUILE */
 static void
 save_pattern_map (Pattern *pat, FILE *file)
 {
