@@ -1,4 +1,4 @@
-/* $Header: /cvs/gnome/gIDE/src/gI_menus.h,v 1.3 1999/12/04 07:11:25 jpr Exp $ */
+/* $Header: /cvs/gnome/gIDE/src/gI_menus.h,v 1.6 2000/02/21 19:55:37 jpr Exp $ */
 /* gIDE
  * Copyright (C) 1998-2000 Steffen Kern
  *
@@ -20,17 +20,26 @@
 #ifndef GI_MENUS_H
 #define GI_MENUS_H
 
-#include <gtk/gtk.h>
+#include <gnome.h>
+#include "gI_window.h"
 
 /*
  * Prototypes for 'gI_menus.c'
  */
-void create_menubar( GtkWidget *app, GtkWidget *vbox );
-GtkWidget *menus_get_item_widget( gchar *path );
-void menus_add_item_widget( gchar *path, GtkWidget *widget);
-void menus_remove_item_widget( gchar *path );
-void menus_set_sensitive( gchar *path, gint sensitive );
-void menus_free_items( void );
+void gI_menus_create_menubar( GideWindow *window );
+
+void gI_menus_set_sensitive( GideWindow *window, gchar *path, gint sensitive );
+void gI_menus_set_sensitive_default( GideWindow *window  );
+void gI_menus_set_sensitive_readonly( GideWindow *window, gboolean state );
+void gI_menus_set_sensitive_debugon( GideWindow *window );
+void gI_menus_set_sensitive_debugoff( GideWindow *window );
+
+void gI_menus_add_item_widget( GideWindow *window, gchar *path,
+			       GtkWidget *widget);
+void gI_menus_remove_item_widget( GideWindow *window, gchar *path );
+GtkWidget* gI_menus_get_item_widget( GideWindow *window, gchar *path );
+
+void gI_menus_free_items( GideWindow *window );
 
 #endif
 
