@@ -1,4 +1,4 @@
-/* $Header: /cvs/gnome/gIDE/src/gI_prefs.c,v 1.7 1999/12/05 19:10:48 jpr Exp $ */
+/* $Header: /cvs/gnome/gIDE/src/gI_prefs.c,v 1.16 2000/04/22 17:23:01 jpr Exp $ */
 /* gIDE
  * Copyright (C) 1998-2000 Steffen Kern
  *
@@ -33,7 +33,7 @@
 #include "gI_document.h"
 #include "gI_toolbar.h"
 
-#ifdef HAVE_LIBGUILE
+#ifdef HAVE_GUILE
 #include <guile/gh.h>
 #endif
 
@@ -42,9 +42,6 @@
 #include "gI_hilite.h"
 #endif
 
-/* prototype */
-void set_cfg_defaults(void);
-
 
 /* globals */
 static GtkWidget *prefs_window = NULL;
@@ -96,9 +93,7 @@ static glong kz_redit = 0;
 
 
 /* externs */
-extern gI_window *main_window;
 extern gchar gide_path[];
-extern gI_config *cfg;
 
 
 /* local prototypes */
@@ -171,42 +166,42 @@ static void cb_autosave_clicked( GtkWidget *widget, gpointer data )
 }
 
 
-static void _update_documents( gI_document *document, gpointer data )
+static void _update_documents( GideDocument *document, gpointer data )
 {
 	if( !document )
 		return;
 
-	gI_text_set_word_wrap( document->text, cfg->wordwrap );
+	gI_text_set_word_wrap( document, cfg->wordwrap );
 
 #ifdef HAVE_GTKTEXT_PATCH
 	if( cfg->highlight )
 	{
-		gtk_editor_hilite_when_idle( GTK_EDITOR( document->text ), TRUE );
+		gtk_editor_hilite_when_idle( GTK_EDITOR( document ), TRUE );
 	}
 	else
 	{
-		gtk_editor_hilite_when_idle( GTK_EDITOR( document->text ), FALSE );
+		gtk_editor_hilite_when_idle( GTK_EDITOR( document ), FALSE );
 	}
 	if( !cfg->edtab )
 	{
-		GTK_EDITOR( document->text )->one_line_ind = NULL;
+		GTK_EDITOR( document )->one_line_ind = NULL;
 	}
 #endif
 
-	gI_text_default_tab_width( document->text ) = cfg->tab_width;
-	gI_text_tab_stops( document->text ) = NULL;
-	gI_text_tab_stops( document->text ) = g_list_prepend( gI_text_tab_stops( document->text ), (void*)cfg->tab_width );
-	gI_text_tab_stops( document->text ) = g_list_prepend( gI_text_tab_stops( document->text ), (void*)cfg->tab_width );
+	gI_text_default_tab_width( document ) = cfg->tab_width;
+	gI_text_tab_stops( document ) = NULL;
+	gI_text_tab_stops( document ) = g_list_prepend( gI_text_tab_stops( document ), (void*)cfg->tab_width );
+	gI_text_tab_stops( document ) = g_list_prepend( gI_text_tab_stops( document ), (void*)cfg->tab_width );
 }
 
 
-static void update_documents( gI_window *window )
+static void update_documents( GideWindow *window )
 {
 	g_list_foreach( window->documents, (GFunc) _update_documents, NULL );
 }
 
 
-static void update_autosave( gI_window *window )
+static void update_autosave( GideWindow *window )
 {
 	if( cfg->autosave )
 	{
@@ -228,7 +223,7 @@ static void update_autosave( gI_window *window )
 }
 
 
-static void update_window( gI_window *window )
+static void update_window( GideWindow *window )
 {
 	/* generate tree for current project */
 	if( cfg->prjftree )
@@ -251,8 +246,8 @@ static void update_window( gI_window *window )
 
 			gtk_widget_show( window->tree_box );
 
-			create_prjftree( window );
-			project_files_tree( window->tree_viewport, project );
+			gI_window_create_prjftree( window );
+			gI_project_files_tree( window->tree_viewport, project );
 		}
 	}
 
@@ -270,17 +265,17 @@ static void update_window( gI_window *window )
 				gdk_font_unref( cfg->style->font );
 			cfg->style->font = new_font;
 		}
-		update_documents_style( main_window );
+		update_documents_style( );
 	}
 
-	gI_create_toolbar( window->window, window->toolbar_box );
+	gI_toolbar_create( GTK_WIDGET(window) );
 	update_documents( window );
 	update_autosave( window );
 }
 
 
 
-#ifdef HAVE_LIBGUILE
+#ifdef HAVE_GUILE
 static glong save_prefs()
 {
 	FILE *config, *backup;
@@ -441,7 +436,7 @@ static glong save_prefs()
 
 	return 1;
 }
-#else  /* HAVE_LIBGUILE */
+#else  /* HAVE_GUILE */
 static glong save_prefs()
 {
 	FILE *config;
@@ -521,7 +516,7 @@ static glong save_prefs()
 
 	return( 1 );
 }
-#endif /* HAVE_LIBGUILE */
+#endif /* HAVE_GUILE */
 
 
 static void prefs_ok_select( GtkWidget *widget, gpointer data )
@@ -883,12 +878,13 @@ void show_preferences( GtkWidget *widget, gpointer data )
 	GtkWidget *label;
 	GtkWidget *book_vbox;
 	GtkWidget *frame;
+	GtkWidget* scrolled;
 	GList *list = NULL;
 	glong i;
 	gchar str[16];
 	gchar *insrow[1];
 	static gchar *list_items[] = {
-	    "New", "Open", "Reload", "Close", "Save", "Print", "Exit", "Undo", "Cut", "Copy",
+	    "New", "Open", "Revert", "Close", "Save", "Save All", "Print", "Exit", "Undo", "Cut", "Copy",
 	    "Paste", "Search", "Preferences", "Help", "Space" };
 	static glong nlist_items = sizeof( list_items ) / sizeof( list_items[0] );
 #ifdef HAVE_GTKTEXT_PATCH
@@ -1308,13 +1304,20 @@ void show_preferences( GtkWidget *widget, gpointer data )
 	gtk_box_pack_start( GTK_BOX( hbox ), label, FALSE, TRUE, 5 );
 	gtk_widget_show( label );
 
+	scrolled = gtk_scrolled_window_new(NULL, NULL);
+	gtk_box_pack_start(GTK_BOX(hbox), scrolled, FALSE, TRUE, 5);
+	gtk_widget_show(scrolled);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
+		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+
 	dest_list = gtk_clist_new( 1 );
-	gtk_box_pack_start( GTK_BOX( hbox ), dest_list, FALSE, TRUE, 5 );
+	gtk_container_add(GTK_CONTAINER(scrolled), dest_list);
+
 	gtk_signal_connect( GTK_OBJECT( dest_list ), "select_row",
 	                    GTK_SIGNAL_FUNC( dest_list_select ), NULL );
 	gtk_signal_connect( GTK_OBJECT( dest_list ), "unselect_row",
 	                    GTK_SIGNAL_FUNC( dest_list_select ), NULL );
-	gtk_widget_set_usize( dest_list, 125, 75 );
+	gtk_widget_set_usize( scrolled, 125, 75 );
 	gtk_widget_show( dest_list );
 
 	c_tbtype = gtk_combo_new();
@@ -1661,7 +1664,7 @@ void set_to_defaults( GtkWidget *widget, gpointer data )
 
 void reload_config( GtkWidget *widget, gpointer data )
 {
-#ifdef HAVE_LIBGUILE
+#ifdef HAVE_GUILE
 	gh_eval_file_with_standard_handler ( cfg->cfgfile );
 #else
 	gI_cf_parse( cfg->cfgfile );
@@ -2273,3 +2276,6 @@ static void update_compile_set( gchar *name, gI_comp_dialog *cpd )
 }
 
 
+
+
+
