@@ -1,4 +1,4 @@
-/* $Header: /cvs/gnome/gIDE/src/gI_prefs.h,v 1.2 1999/11/06 17:45:39 jpr Exp $ */
+/* $Header: /cvs/gnome/gIDE/src/gI_prefs.h,v 1.3 2000/04/19 12:51:32 jpr Exp $ */
 /* gIDE
  * Copyright (C) 1998-2000 Steffen Kern
  *
@@ -28,7 +28,7 @@
 void show_preferences( GtkWidget *widget, gpointer data );
 void set_to_defaults( GtkWidget *widget, gpointer data );
 void reload_config( GtkWidget *widget, gpointer data );
-gI_comp_dialog *gI_comp_dialog_new();
+gI_comp_dialog *gI_comp_dialog_new(void);
 gI_comp_set *gI_comp_set_get( gchar *cset );
 
 #endif
