@@ -1,4 +1,4 @@
-/* $Header: /cvs/gnome/gIDE/src/gI_toolbar.h,v 1.1 1999/12/04 03:00:24 jpr Exp $ */
+/* $Header: /cvs/gnome/gIDE/src/gI_toolbar.h,v 1.2 2000/04/14 04:34:00 jpr Exp $ */
 /* gIDE
  * Copyright (C) 1998-2000 Steffen Kern
  *
@@ -25,8 +25,7 @@
 /*
  * Prototypes for 'gI_toolbar.c'
  */
-
-void gI_create_toolbar( GtkWidget *app, GtkWidget *vbox );
+void gI_toolbar_create(GtkWidget* app);
 
 #endif
 
