@@ -1,4 +1,4 @@
-/* $Header: /cvs/gnome/gIDE/src/gide.h,v 1.2 1999/11/06 17:45:39 jpr Exp $ */
+/* $Header: /cvs/gnome/gIDE/src/gide.h,v 1.6 2000/04/22 17:23:01 jpr Exp $ */
 /* gIDE
  * Copyright (C) 1998-2000 Steffen Kern
  *
@@ -20,38 +20,22 @@
 #ifndef GIDE_H
 #define GIDE_H
 
-#include <gtk/gtk.h>
+#include <gnome.h>
+#include "gI_window.h"
+#include "gI_document.h"
 #include "structs.h"
 
-/* Header-File for gide.c */
-
 /* Widgets: Main-Window & Box */
-extern gI_window *main_window;
-extern GtkWidget *main_vbox;
-
-/* Widgets: Boxes */
-extern GtkWidget *main_hbox1;
-extern GtkWidget *main_hbox2;
-extern GtkWidget *main_hbox3;
-
-/* Widgets: misc */
-extern GtkWidget *main_notebook;
-extern GtkWidget *main_text1;
-extern GtkWidget *main_frame;
-extern GtkWidget *main_vscrollbar;
-extern GtkWidget *main_hscrollbar;
-extern GtkWidget *main_statusbar;
-extern GtkWidget *main_table;
+extern GideWindow *main_window;
 
 /* Configuration */
 extern gI_config *cfg;
 
-extern char *home;
 extern char gide_path[MAXLEN];
 extern char tools_path[MAXLEN];
 extern char prj_path[MAXLEN];
 
-void free_op_list( GList *op_list );
+void set_cfg_defaults(void);
 
 #endif
 
